Package: LISTO
Type: Package
Title: Performing Comprehensive Overlap Assessments
Version: 0.7.3
Authors@R: person('Andrei-Florian', 'Stoica', 
      email='andreistoica@foxmail.com',
     role=c('aut','cre'),
     comment=c(ORCID='0000-0002-5253-0826'))
Description: The implementation of a statistical framework for performing overlap assessments on
    lists comprising sets of strings (such as lists of gene sets) described in Stoica (2023)
    <https://ora.ox.ac.uk/objects/uuid:b0847284-a02f-47ee-88e3-a3c4e0cdb8b1>.
    It can assess overlaps of pairs of sets of strings selected either from the same universe or 
    from different universes, and overlaps of triplets of sets of strings selected from the same
    universe. Designed for single-cell RNA-sequencing data analysis applications, but suitable
    for other purposes as well.
License: MIT + file LICENSE
Imports: methods, parallel, primes, statisfactory, stats
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: qs2, scRNAseq, scuttle, Seurat, testthat (>= 3.0.0), withr
URL: https://github.com/andrei-stoica26/LISTO
BugReports: https://github.com/andrei-stoica26/LISTO/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-25 13:21:39 UTC; Andrei
Author: Andrei-Florian Stoica [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5253-0826>)
Maintainer: Andrei-Florian Stoica <andreistoica@foxmail.com>
Repository: CRAN
Date/Publication: 2026-04-25 13:40:02 UTC
Built: R 4.5.2; ; 2026-04-26 23:55:49 UTC; unix

# Generated by roxygen2: do not edit by hand

export(buildSeuratMarkerList)
export(factorialPrimePowers)
export(mtCorrectDF)
export(mtCorrectV)
export(probCounts2MN)
export(probCounts3N)
export(pvalCounts2MN)
export(pvalCounts3N)
export(pvalObjects)
export(pvalSets2MN)
export(pvalSets2N)
export(pvalSets3N)
export(runLISTO)
export(vChoose)
export(vSum)
importFrom(methods,is)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,parSapply)
importFrom(parallel,stopCluster)
importFrom(primes,generate_n_primes)
importFrom(primes,generate_primes)
importFrom(statisfactory,psum)
importFrom(stats,dhyper)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,phyper)

# listo

Listo is an enhanced version of the `list` type that aims to do the following:

- Make behavior more consistent. No more methods with "in-place" changes. Every method call returns a value
- Adds methods that are missing from the `list` type. For example, `listo` has a `first` method that returns `self[0]`
- A list of arguments passed in will be converted to a `listo` object, but a single argument that is an iterator will be unpacked into a `listo` object. This is useful for when you want to pass in a generator or other iterator into a function that expects a list

## Usage

First install it via pip, poetry, or whatever you use

```bash
pip install listo
```

Then use it in your code

```python
from listo import listo

lst = listo(1, 2, 3)
assert lst.first() == 1
assert lst.last() == 2
```

### Usage: How listo handles initial arguments

For arguments passed in, it just converts them to a `listo` object

```python
lst = listo(1, 2, 3)
assert lst == [1, 2, 3]

lst2 = listo([1, 2, 3]) # list is unpacked
assert lst2 == [1, 2, 3]

lst3 = listo((1, 2, 3)) # tuple is converted and unpacked
assert lst3 == [1, 2, 3]

lst4 = listo([1, 2, 3], (1, 2))  # Two args mean nothing is unpacked
assert lst4 == [[1, 2, 3], (1, 2)]
```

This is different from standard Python iterator behavior.

## Development

Install the package in editable mode with test dependencies:

```bash
pip install -e '.[test]'
```

### Code quality

```bash 
make lint
```

### Testing

```bash
make test
```

### Releasing on PyPI

1. Update the `version` in `pyproject.toml`. We use semantic versioning
2. Create and merge a PR branch called `release-x.x.x`
3. Pull from `main``
4. At the command line, run `make tag`
5. Go to [tags page](https://github.com/pydanny/listo/tags), choose the most recent tag, and click `Draft a new release`


### Building the project locally

Go to the project root

```bash
pip install --upgrade build
python -m build
```

Test the project, forcing reinstall if necessary

```bash
pip install dist/listo-0.1.0-py3-none-any.whl --force-reinstall
```

# LISTO

`LISTO` is a tool for performing comprehensive overlap assessments
on lists comprising sets of strings, such as lists of gene sets. It can 
assess:

- Overlaps of pairs of sets of strings selected from the same
universe.
- Overlaps of pairs of sets of strings selected from different universes.
- Overlaps of triplets of sets of strings selected from the same universe. 

While `LISTO` has been developed with scRNA-seq data analysis in mind, 
the methodology is fully applicable for the same problem arising in any other 
setting. Thus, the implementation of `LISTO` uses general R objects 
(data frames, character vectors), rather than scRNA-seq-specific objects.

## Installation

To install the version of `LISTO` currently available on CRAN, run the 
following R code:

```r
install.packages("LISTO")
```

Alternatively, you can install the most recent development version using 
this code: 

```r
pak::pak("andrei-stoica26/LISTO")

```

Currently, both version are the same (`0.7.3`).

## Description and usage

This section will elaborate on the functionality and usage of `LISTO`. It 
discusses first the overlaps of individual elements, then the details of how
the lists of elements must be provided as input.


### Items

Each item taking part in an individual overlap assessed by `LISTO` is a 
set of strings. Each overlap assessment of sets of strings answers the 
question of whether the sets intersect each other to a statistically 
significant extent.

### Lists

The `runLISTO` function runs the entire LISTO pipeline. It requires two lists
as input. Each list can store two types of elements:

- Character vectors.
- Data frames with a numeric column specified by the `numCol` parameter. 

A third list, containing the same type of elements, can be optionally provided.

### Extracting items from lists

Items to be used in the overlap assessments are extracted from the input
lists as follows:

- Character vectors: They are used as such.

- Data frames: The rownames of the data frame are selected, and overlaps
are calculated based on cutoffs determined by the distinct values in the
column specified by `numCol`. The median of the resulting p-values 
is taken to be the p-value of the corresponding overlap.

# A self-contained script to generate the data in extdata
#
createExtData <- function(){
    if (requireNamespace(c('LISTO', 'qs2', 'scRNAseq','scuttle', 'Seurat',
                           'withr'), quietly=TRUE)){
        scObj <- scRNAseq::BaronPancreasData('human')
        scObj <- scuttle::logNormCounts(scObj)
        scObj <- Seurat::as.Seurat(scObj)

        donorMarkers <- LISTO::buildSeuratMarkerList(scObj, 'donor')
        labelMarkers <- LISTO::buildSeuratMarkerList(scObj, 'label')
        signatures <- list(
            acinarMarkers = c('PRSS1', 'KLK1', 'CTRC', 'PNLIP', 'AKR1C3',
                              'CTRB1','DUOXA2', 'ALDOB', 'REG3A', 'SERPINA3',
                              'PRSS3', 'REG1B','CFB',	'GDF15', 'MUC1',
                              'ANPEP', 'ANGPTL4', 'OLFM4','GSTA1', 'LGALS2',
                              'PDZK1IP1', 'RARRES2','CXCL17','UBD', 'GSTA2',
                              'LYZ', 'RBPJL', 'PTF1A', 'CELA3A','SPINK1',
                              'ZG16', 'CEL', 'CELA2A', 'CPB1', 'CELA1',
                              'PNLIPRP1', 'RNASE1', 'AMY2B', 'CPA2','CPA1',
                              'CELA3B','CTRB2', 'PLA2G1B', 'PRSS2', 'CLPS',
                              'REG1A', 'SYCN'),
            alphaMarkers = c('GCG', 'TTR', 'PCSK2', 'FXYD5', 'LDB2', 'MAFB',
                          'CHGA', 'SCGB2A1', 'GLS', 'FAP', 'DPP4', 'GPR119',
                          'PAX6', 'NEUROD1', 'LOXL4', 'PLCE1', 'GC', 'KLHL41',
                          'FEV', 'PTGER3', 'RFX6', 'SMARCA1', 'PGR', 'IRX1',
                          'UCP2', 'RGS4', 'KCNK16', 'GLP1R', 'ARX', 'POU3F4',
                          'RESP18', 'PYY', 'SLC38A5', 'TM4SF4', 'CRYBA2',
                          'SH3GL2','PCSK1', 'PRRG2', 'IRX2', 'ALDH1A1','PEMT',
                          'SMIM24','F10', 'SCGN', 'SLC30A8'),
            betaMarkers = c('INS', 'IAPP', 'GJD2', 'PDX1', 'SLC2A2', 'NPY',
                            'MAFA', 'PFKFB2','HOPX', 'PAX6', 'MAFB', 'CASR',
                            'EDARADD', 'SCGB2A1','TGFBR3','ADCYAP1', 'SH3GL2',
                            'NEUROD1', 'ISL1', 'RGS16','SMAD9', 'SIX3','BMP5',
                            'PIR', 'STXBP5', 'DLK1', 'MEG3','GCGR', 'LMX1A',
                            'JPH3', 'CD40', 'HAMP','EZH1', 'NTRK1', 'FXYD2',
                            'RIMS1', 'EFNA5', 'NPTX2', 'PAX4', 'PCSK2',
                            'G6PC2', 'SLC30A8', 'PCSK1', 'SCGN', 'IGF2',
                            'SYT13', 'FFAR2', 'SIX2'),
            deltaMarkers = c('SST', 'GABRB3', 'FRZB', 'MS4A8', 'BAIAP3',
                             'CASR', 'BCHE', 'UNC5B','EDN3', 'GHSR', 'PCSK1',
                             'GABRG2', 'POU3F1', 'BHLHE41', 'EHF', 'LCORL',
                             'ETV1', 'PDX1', 'LEPR', 'UCP2', 'NPTX2', 'FXYD2',
                             'IAPP', 'KCNK16', 'SCGN', 'ISL1', 'HHEX',
                             'RESP18', 'PAX4', 'RBP4', 'PCSK9', 'FFAR4'),
            ductalMarkers =  c('CFTR', 'SERPINA5', 'SLPI', 'TFF1', 'CFB',
                               'LGALS4', 'CTSH',	'PERP', 'PDLIM3', 'WFDC2',
                               'SLC3A1', 'AQP1', 'ALDH1A3', 'VTCN1', 'KRT19',
                               'TFF2', 'KRT7', 'CLDN4', 'LAMB3', 'TACSTD2',
                               'CCL2', 'DCDC2','CXCL2', 'CLDN10', 'HNF1B',
                               'KRT20', 'MUC1', 'ONECUT1', 'AMBP', 'HHEX',
                               'ANXA4', 'SPP1', 'PDX1', 'SERPINA3', 'GDF15',
                               'AKR1C3', 'MMP7', 'DEFB1', 'SERPING1',
                               'TSPAN8', 'CLDN1', 'S100A10', 'PIGR'),
            gammaMarkers = c('PPY', 'ABCC9', 'FGB', 'ZNF503', 'MEIS1', 'LMO3',
                             'EGR3', 'CHN2', 'PTGFR', 'ENTPD2', 'AQP3',
                             'THSD7A', 'CARTPT', 'ISL1', 'PAX6', 'NEUROD1',
                             'APOBEC2', 'SEMA3E', 'SLITRK6','SERTM1', 'PXK',
                             'PPY2P', 'ETV1', 'ARX', 'CMTM8', 'SCGB2A1',
                             'FXYD2', 'SCGN'))

        qs2::qs_save(donorMarkers, 'inst/extdata/donorMarkers.qs2')
        qs2::qs_save(labelMarkers, 'inst/extdata/labelMarkers.qs2')
        qs2::qs_save(signatures, 'inst/extdata/signatures.qs2')
        qs2::qs_save(rownames(scObj), 'inst/extdata/universe1.qs2')
        qs2::qs_save(Reduce(union, signatures), 'inst/extdata/universe2.qs2')

        seuratObj <- withr::with_seed(1, scuttle::mockSCE(ngenes=500))
        seuratObj <- Seurat::as.Seurat(seuratObj, data=NULL)
        seuratObj <- Seurat::NormalizeData(seuratObj)
        qs2::qs_save(seuratObj, 'inst/extdata/seuratObj.qs2')
    }
}

createExtData()

#' Add numeric vectors of different lenghts
#'
#' This function adds numeric vectors of different lengths by filling shorter
#' vectors with zeroes.
#'
#' @param ... Numeric vectors.
#'
#' @return A numeric vector.
#'
#' @examples
#' vSum(c(1, 4), c(2, 8, 6), c(1, 7), c(10, 4, 6, 7))
#'
#' @export
#'
vSum <- function(...){
    vectors <- list(...)
    lengths <- vapply(vectors, length, integer(1))
    maxLen <- max(lengths)
    vectors <- mapply(function(v, l) c(v, rep(0, maxLen - l)), vectors,
                      lengths, SIMPLIFY=FALSE)
    return(psum(do.call(psum, vectors)))
}

#' Compute the prime factor decomposition of the binomial coefficient
#'
#' This function computes the prime factor decomposition of the
#' binomial coefficient.
#'
#' @param n Total number of elements.
#' @param k Number of selected elements.
#'
#' @return A vector in which positions represent prime numbers (that is, the
#' first position corresponds to 2, the second position corresponds to 3,
#' the third position corresponds to 5, etc.) and values
#' represent their exponents in the factorial decomposition.
#'
#' @examples
#' vChoose(8, 4)
#'
#' @export
#'
vChoose <- function(n, k)
    return(vSum(factorialPrimePowers(n),
                -1 * factorialPrimePowers(k),
                -1 * factorialPrimePowers(n - k)))

#' Compute the prime representation of the numerator of the fraction
#' representing the probability that two subsets of sets M and N intersect
#' in k points
#'
#' This function computes the numerator of the fraction representing the
#' probability that two subsets of sets M and N intersect in k points
#'
#' @param intMN Number of elements in the intersection of sets M and N.
#' @param intAN Number of elements in the intersection of sets A (subset of M)
#' and N.
#' @param intBM Number of elements in the intersection of sets B (subset of N)
#' and M.
#' @param k Number of elements in the intersection of sets A and B.
#'
#' @return A vector containing the prime representation of the fraction
#' representing the probability that two subsets of sets M and N intersect in k
#' points. Positions represent prime numbers in order (2, 3, 5...), and values
#' represent their exponents in the prime decomposition.
#'
#' @keywords internal
#'
vNumeratorMN <- function(intMN, intAN, intBM, k)
    return(vSum(vChoose(intAN, k), vChoose(intMN - intAN, intBM - k)))

test_that("checkNumColAll works", {
    df1 <- data.frame(fruit = c('apple', 'banana', 'cherry'),
                      cost = c(6, 5, 3))
    df2 <- data.frame(fruit = c('apple', 'banana', 'cherry'),
                      cost = c('6', '5', '3'))

    expect_no_message(checkNumCol(df1, 'cost'))
    expect_error(checkNumCol(df1, 'price'))
    expect_error(checkNumCol(df2, 'cost'))
    expect_no_message(checkNumCol(LETTERS, 'cost'))

    expect_no_message(checkNumColAll (list(df1, LETTERS), 'cost'))
    expect_error(checkNumColAll (list(df1, LETTERS), 'price'))
    expect_error(checkNumColAll (list(df2, LETTERS), 'cost'))
    expect_no_message(checkNumColAll (list(LETTERS, LETTERS), 'cost'))
})

test_that("getObjectValues works", {
    df <- data.frame(fruit = c('apple', 'banana', 'cherry'),
                      cost = c(6, 5, 3))
    expect_identical(getObjectValues(df, 'cost'), c(6, 5, 3))
    expect_identical(getObjectValues(df, NULL), -Inf)
    expect_identical(getObjectValues(LETTERS, 'cost', FALSE), Inf)
})

test_that("generateCutoffs works", {
    res <- generateCutoffs(donorMarkers[[1]],
                           donorMarkers[[2]],
                           numCol='avg_log2FC')
    expect_equal(res[1], 2.743160, tolerance=0.0001)
    res <- generateCutoffs(donorMarkers[[2]],
                           donorMarkers[[3]],
                           labelMarkers[[1]],
                           numCol='avg_log2FC')
    expect_equal(res[1], 3.501303, tolerance=0.0001)

})

test_that("factorization functions work", {
    expect_identical(factorialPrimePowers(8), c(7, 2, 1, 1))
    expect_identical(powerProduct(c(2, 3, 5), c(4, 2, 6)), 2250000)
})

test_that("multiple testing functions work", {
    pvals <- c(0.032, 0.001, 0.0045, 0.051, 0.048, 0.33)
    res <- mtCorrectV(pvals)
    expect_equal(res, c(0.149940, 0.014700, 0.033075, 0.149940, 0.149940,
                        0.808500), tolerance=0.0001)
    res <- mtCorrectV(pvals, 'hochberg', 'mean')
    expect_equal(res, 0.11075, tolerance=0.0001)
    res <- mtCorrectV(pvals, 'bonferroni', nComp=4)
    expect_equal(res, c(0.128, 0.004, 0.018, 0.204, 0.192, 1.000))
    df <- data.frame(elem = c(LETTERS[seq(6)]),
    pval = pvals)
    res <- mtCorrectDF(df)
    expect_equal(res$pvalAdj, c(0.014700, 0.033075), tolerance=0.0001)
    res <- mtCorrectDF(df, 'bonferroni', nComp=3)
    expect_equal(res$pvalAdj, c(0.0030, 0.0135), tolerance=0.0001)
    expect_error(mtCorrectDF(df, 'holm', nComp=3))
})

test_that("probCounts2MN works", {
    res <- probCounts2MN(300, 70, 110, 6)
    expect_equal(res, 2.091706e-09, tolerance=0.0001)
    res <- sum(vapply(seq(0, 70), function(i)
        probCounts2MN(300, 70, 110, i), numeric(1)))
    expect_equal(res, 1, tolerance=0.0001)
})

test_that("probCounts3N works", {
    res <- probCounts3N(25, 20, 30, 140, 8)
    expect_equal(res, 6.875155e-08, tolerance=0.0001)
    res <- sum(vapply(seq(0, 20), function(k) probCounts3N(25, 20, 30, 140, k),
                      numeric(1)))
    expect_equal(res, 1, tolerance=0.0001)
})

test_that("pvalCounts functions work", {
    expect_equal(pvalCounts2MN(300, 23, 24, 6), 0.005571074,
                 tolerance=0.0001)
    expect_equal(pvalCounts3N(300, 200, 250, 400, 180), 5.101079e-62,
                               tolerance=0.0001)
})

test_that("pvalObjects works", {
    res <- pvalObjects(LETTERS[seq(2, 7)],
                       LETTERS[seq(3, 19)],
                       universe1=LETTERS)
    expect_equal(res, 0.2956522, tolerance=0.0001)
    res <- pvalObjects(LETTERS[seq(2, 7)],
                       LETTERS[seq(3, 19)],
                       LETTERS[seq(4, 8)],
                       universe1=LETTERS,
                       type='3N')
    expect_equal(res, 0.0007643267, tolerance=0.0001)
    res <- pvalObjects(LETTERS[seq(2, 7)],
                       LETTERS[seq(3, 8)],
                       universe1=LETTERS[seq(1, 16)],
                       universe2=LETTERS[seq(2, 26)],
                       type='2MN')
    expect_equal(res, 0.01098901, tolerance=0.0001)
})

test_that("pvalSets functions work", {
    res <- pvalSets2N(LETTERS[seq(4, 10)], LETTERS[seq(7, 15)], LETTERS)
    expect_equal(res, 0.1585284, tolerance=0.0001)
    res <- pvalSets2MN(LETTERS[seq(4, 10)],
                       LETTERS[seq(7, 15)],
                       LETTERS[seq(19)],
                       LETTERS[seq(6, 26)])
    expect_equal(res, 0.3776224, tolerance=0.0001)
    res <- pvalSets3N(LETTERS[seq(4, 10)],
                      LETTERS[seq(7, 15)],
                      LETTERS[seq(19)],
                      LETTERS)
    expect_equal(res, 0.05096209, tolerance=0.0001)
})

test_that("runLISTO works", {
    res <-  runLISTO(donorMarkers, labelMarkers, universe1=universe1,
                     numCol='avg_log2FC', verbose=FALSE)
    expect_equal(dim(res), c(length(donorMarkers) * length(labelMarkers), 4))
    expect_equal(res[1, 4], 1.636216e-51, tolerance=0.0001)
    res <- runLISTO(donorMarkers, signatures, universe1=universe1,
                    universe2=universe2, numCol='avg_log2FC',
                    verbose=FALSE)
    expect_equal(dim(res), c(length(donorMarkers) * length(signatures), 4))
    expect_equal(res[1, 4], 9.317070e-41, tolerance=0.0001)
    res <-  runLISTO(donorMarkers, labelMarkers, signatures,
                     universe1=universe1, numCol='avg_log2FC',
                     verbose=FALSE)
    expect_equal(dim(res), c(length(donorMarkers) *
                                 length(labelMarkers) *
                                 length(signatures), 5))
    expect_equal(res[1, 4], 1.259111e-86, tolerance=0.0001)
})

test_that("vector functions work", {
    res <- vSum(c(1, 4), c(2, 8, 6), c(1, 7), c(10, 4, 6, 7))
    expect_identical(res, c(14, 23, 12, 7))
    res <- vChoose(8, 4)
    expect_identical(res, c(1, 0, 1, 1))
    res <- vNumeratorMN(20, 8, 6, 2)
    expect_identical(res, c(2, 2, 1, 1, 1))
})

test_that("buildSeuratMarkerList works", {
    res <- buildSeuratMarkerList(seuratObj, 'Cell_Cycle', logFCThr=0.1)
    expect_true(is(res$G0, 'data.frame'))
    expect_equal(res$G0$p_val_adj[1], 0.8366238, tolerance=0.0001)
    expect_equal(length(res), 4)
})

#' @importFrom methods is
#' @importFrom parallel clusterExport makeCluster parSapply stopCluster
#'
NULL

#' Filter items based on a provided cutoff
#'
#' This function filters items based on a provided cutoff.
#'
#' @inheritParams getObjectValues
#' @param cutoff Cutoff for assessing item overlaps.
#' @param compFun Comparison function.
#'
#' @keywords internal
#'
filterItems <- function(obj, numCol = NULL, cutoff = NULL, compFun = `>`){
    if (is.null(numCol) | is(obj, 'character'))
        return(obj)
    return(rownames(obj[compFun(obj[[numCol]], cutoff), ]))
}

#' Compute the p-value of overlap for two or three objects
#'
#' This function computes the p-value of overlap for two or three objects.
#'
#' @inheritParams generateCutoffs
#' @param universe1 The set from which the items stored
#' in \code{obj1} are selected.
#' @param universe2 The set from which the items stored
#' in \code{obj2} are selected.
#'
#' @return A p-value.
#'
#' @keywords internal
#'
pvalObjectsCore <- function(obj1,
                            obj2,
                            obj3 = NULL,
                            universe1,
                            universe2 = NULL,
                            numCol = NULL,
                            cutoff = NULL,
                            compFun = `>`,
                            type = c('2N', '2MN', '3N')){

    names1 <- filterItems(obj1, numCol, cutoff, compFun)
    names2 <- filterItems(obj2, numCol, cutoff, compFun)
    if (type == '2N')
        return(pvalSets2N(names1, names2, universe1))

    if (type == '2MN')
        return(pvalSets2MN(names1, names2, universe1, universe2))

    if (type == '3N'){
        names3 <- filterItems(obj3, numCol, cutoff, compFun)
        return(pvalSets3N(names1, names2, names3, universe1))
    }
}

#' Assess the overlap of two or three objects
#'
#' This function assesses the overlap of two or three objects
#' (character vectors, or data frames having a numeric column).
#'
#' @inheritParams generateCutoffs
#' @inheritParams pvalObjectsCore
#' @inheritParams mtCorrectV
#' @param mtMethod Multiple testing correction method.
#' @param nCores Number of cores. If performing an overlap assessment between
#' sets belonging to the same universe, it is recommended not to use
#' parallelization (that is, leave this parameter as 1).
#' @param type Type of overlap assessment. Choose between: two sets belonging
#' to the same universe ('2N'), two sets belonging to different universes
#' ('2MN'), three sets belonging to the same universe ('3MN').
#'
#' @return A numeric value in [0, 1] representing the p-value of the
#' overlap of the two objects.
#'
#' @examples
#' pvalObjects(LETTERS[seq(2, 7)], LETTERS[seq(3, 19)], universe1=LETTERS)
#'
#' @export
#'
pvalObjects <- function(obj1,
                        obj2,
                        obj3 = NULL,
                        universe1,
                        universe2 = NULL,
                        numCol = NULL,
                        isHighTop = TRUE,
                        maxCutoffs = 5000,
                        mtMethod = c('BY', 'holm', 'hochberg',
                                     'hommel', 'bonferroni', 'BH',
                                     'fdr', 'none'),
                        nCores = 1,
                        type = c('2N', '2MN', '3N')){

    checkNumColAll(list(obj1, obj2, obj3), numCol)
    mtMethod <- match.arg(mtMethod, c('BY', 'holm', 'hochberg',
                                      'hommel', 'bonferroni', 'BH',
                                      'fdr', 'none'))
    type <- match.arg(type, c('2N', '2MN', '3N'))

    cutoffs <- generateCutoffs(obj1, obj2, obj3, numCol, isHighTop,
                               maxCutoffs)
    compFun <- ifelse(isHighTop, `>`, `<`)

    if (nCores == 1){
        pvals <- vapply(cutoffs, function(cutoff)
            return(pvalObjectsCore(obj1, obj2, obj3, universe1, universe2,
                                   numCol, cutoff, compFun, type)), numeric(1))

    } else{
        if (type == '2N')
            warning('Parallelization is not recommended for 2N overlap ',
                    'assessments. `nCores` should be set back to 1')
        clust <- makeCluster(nCores)
        clusterExport(clust,
                      c('obj1', 'obj2', 'obj3',  'universe1', 'universe2',
                        'numCol', 'cutoffs', 'compFun', 'type'),
                      envir=environment())
        pvals <- parSapply(clust, cutoffs, function(cutoff)
            return(pvalObjectsCore(obj1, obj2, obj3, universe1, universe2,
                                   numCol, cutoff, compFun, type)))
        stopCluster(clust)
    }

    pval <- mtCorrectV(pvals, mtMethod, 'median')
    return(pval)
}

#' Generate the prime factor decomposition of n factorial.
#'
#' This function generates the prime factor decomposition of n factorial.
#'
#' @param n A positive integer.
#'
#' @return A vector in which positions represent prime numbers (that is, the
#' first position corresponds to 2, the second position corresponds to 3,
#' the third position corresponds to 5, etc.) and values
#' represent their exponents in the factorial decomposition.
#'
#' @examples
#' factorialPrimePowers(8)
#'
#' @export
#'
factorialPrimePowers <- function(n){
    if (n %in% c(0, 1))
        return(NULL)
    if(n < 0)
        stop('`n` must be a non-negative integer.')
    primes <- generate_primes(max=n)
    nPrimes <- length(primes)
    result <- rep(0, nPrimes)
    for (i in seq(nPrimes)){
        k <- primes[i]
        while (k <= n){
            result[i] <- result[i] + floor(n / k)
            k <- k * primes[i]
        }
    }
    return(result)
}

#' Compute the prime factor decomposition of the binomial coefficient
#'
#' This function computes the prime factor decomposition of the
#' binomial coefficient.
#'
#' @param n Total number of elements.
#' @param k Number of selected elements.
#'
#' @return The product of the primes raised to the exponents.
#'
#' @examples
#' powerProduct(c(2, 3, 5), c(4, 2, 6))
#'
#' @noRd
#'
powerProduct <- function(primes, exponents)
    return(prod(mapply(function(x, y) x ^ y, primes, exponents)))

#' @importFrom stats dhyper median p.adjust phyper
#'
NULL

#' Perform Bonferroni correction on a vector of p-values
#'
#' This function performs Bonferroni correction on a vector of p-values.
#'
#' @details This function is implemented in order to allow users to perform
#' Bonferroni correction with fewer comparisons than the number of elements
#' in the vector, which is normally disallowed by the \code{p.adjust} function
#' from \code{stats}. A use case is correcting the Seurat markers returned
#' for each identity class by setting the number of comparisons to the number
#' of classes.
#'
#' @param pvals A numeric vector.
#' @param nComp Number of comparisons. In most situations, this parameter
#' should not be changed.
#'
#' @return Bonferroni-corrected p-values
#'
#' @keywords internal
#'
bfCorrectV <- function(pvals, nComp)
    return(vapply(pvals, function(x) min (x * nComp, 1), numeric(1)))


#' Helper function for multiple comparison testing
#'
#' This function is a helper for multiple comparison testing.
#'
#' @inheritParams bfCorrectV
#' @param mtMethod Multiple testing correction method. Choices are
#' 'BY' (default) 'holm', hochberg', hommel', 'bonferroni', 'BH',  'fdr' and
#' 'none'.
#'
#' @return Adjusted p-values.
#'
#' @keywords internal
#'
mtCorrectHelper <- function(pvals, mtMethod, nComp){
    if (nComp >= length(pvals))
        return(p.adjust(pvals, mtMethod, nComp))

    if (mtMethod == 'bonferroni')
        return(bfCorrectV(pvals, nComp))

    stop('`nComp` can be greater than the number of rows ',
         'only if `mtMethod` is set to bonferroni.')
}

#' Perform multiple testing correction on a vector of p-values
#'
#' This function performs multiple testing correction on a vector of p-values.
#'
#' @inheritParams mtCorrectHelper
#' @param mtMethod Multiple testing correction method. Choices are
#' 'BY' (default) 'holm', hochberg', hommel', 'bonferroni', 'BH',  'fdr' and
#' 'none'.
#' @param mtStat A statistics to be optionally computed. Choices are 'identity'
#' (no statistics will be computed and the adjusted p-values will be returned
#' as such), 'median', 'mean', 'max' and 'min'.
#'
#' @return If \code{mtStat} is 'identity' (as default), a numeric vector of
#' p-values corrected for multiple testing. Otherwise, a statistic based on
#' these corrected p-values defined by \code{mtStat}.
#'
#' @examples
#' pvals <- c(0.032, 0.001, 0.0045, 0.051, 0.048)
#' mtCorrectV(pvals)
#'
#' @export
#'
mtCorrectV <- function(pvals,
                       mtMethod = c('BY', 'holm', 'hochberg', 'hommel',
                                    'bonferroni', 'BH', 'fdr', 'none'),
                       mtStat = c('identity', 'median', 'mean', 'max', 'min'),
                       nComp = length(pvals)){

    mtMethod <- match.arg(mtMethod)
    mtStat <- match.arg(mtStat)
    statFun <- eval(as.name(mtStat))
    return(statFun(mtCorrectHelper(pvals, mtMethod, nComp)))
}

#' Perform multiple testing correction on a data frame column
#'
#' This function orders a data frame based on a column of p-values, performs
#' multiple testing on the column, and filters the data-frame based on it.
#'
#' @inheritParams mtCorrectV
#' @param df A data frame with a p-values column.
#' @param colStr Name of the column of p-values.
#' @param newColStr Name of the column of adjusted p-values that will be
#' created.
#' @param doOrder Whether to increasingly order the data frame based on the
#' adjusted p-values.
#' @param pvalThr p-value threshold used for filtering. If \code{NULL}, no
#' filtering will be performed.
#'
#' @return A data frame with the p-value column corrected for multiple testing.
#'
#' @examples
#' df <- data.frame(elem = c('A', 'B', 'C', 'D', 'E'),
#' pval = c(0.032, 0.001, 0.0045, 0.051, 0.048))
#' mtCorrectDF(df)
#'
#' @export
#'
mtCorrectDF <- function(df,
                        mtMethod = c('BY', 'holm', 'hochberg', 'hommel',
                                     'bonferroni', 'BH', 'fdr', 'none'),
                        colStr = 'pval',
                        newColStr = 'pvalAdj',
                        pvalThr = 0.05,
                        doOrder = TRUE,
                        nComp = nrow(df)){
    mtMethod <- match.arg(mtMethod)
    df[[newColStr]] <- mtCorrectHelper(df[[colStr]], mtMethod, nComp)
    if (!is.null(pvalThr))
        df <- df[df[, newColStr] < pvalThr, ]
    if (doOrder)
        df <- df[order(df[[newColStr]]), ]
    return(df)
}

donorPath <- system.file('extdata', 'donorMarkers.qs2', package='LISTO')
donorMarkers <- qs2::qs_read(donorPath)
labelPath <- system.file('extdata', 'labelMarkers.qs2', package='LISTO')
labelMarkers <- qs2::qs_read(labelPath)
sigPath <- system.file('extdata', 'signatures.qs2', package='LISTO')
signatures <- qs2::qs_read(sigPath)
universe1Path <- system.file('extdata', 'universe1.qs2', package='LISTO')
universe1 <- qs2::qs_read(universe1Path)
universe2Path <- system.file('extdata', 'universe2.qs2', package='LISTO')
universe2 <- qs2::qs_read(universe1Path)
seuratPath <- system.file('extdata', 'seuratObj.qs2', package='LISTO')
seuratObj <- qs2::qs_read(seuratPath)

#' Assess the overlap of two or three lists of objects.
#'
#' This function assesses the overlap of two or three lists of objects
#' (character vectors, or data frames having at least one numeric column).
#'
#' @inheritParams pvalObjects
#' @param list1 A list containing character vectors, or data frames having
#' a numeric column.
#' @param list2 A list containing character vectors, or data frames having a
#' numeric column.
#' @param list3 A list containing character vectors, or data frames having a
#' numeric column.
#' @param universe1 Character vector; the set from which the items
#' corresponding to the elements in \code{list1} are selected.
#' @param universe2 Character vector; the set from which the items
#' corresponding to the elements in \code{list2} are selected.
#' @param pvalThr Threshold to filter the results based on the adjusted
#' p-values. If \code{NULL} as default, no filtering will be performed.
#' @param verbose Logical; whether the output should be verbose.
#' @param ... Additional arguments passed to \code{mtCorrectDF}.
#'
#' @return A data frame listing the p-value and adjusted p-value for each
#' overlap. Combinations of overlaps are represented through the first two
#' (or three if \code{list3} is not \code{NULL}) columns, while the penultimate
#' column records the overlap p-values and the last column records the adjusted
#' overlap p-values.
#'
#' @examples
#' donorPath <- system.file('extdata', 'donorMarkers.qs2', package='LISTO')
#' donorMarkers <- qs2::qs_read(donorPath)[seq(3)]
#' labelPath <- system.file('extdata', 'labelMarkers.qs2', package='LISTO')
#' labelMarkers <- qs2::qs_read(labelPath)[seq(3)]
#' universe1Path <- system.file('extdata', 'universe1.qs2', package='LISTO')
#' universe1 <- qs2::qs_read(universe1Path)
#' res <-  runLISTO(donorMarkers, labelMarkers, universe1=universe1,
#' numCol='avg_log2FC')
#'
#' @export
#'
runLISTO <- function(list1,
                     list2,
                     list3 = NULL,
                     universe1,
                     universe2 = NULL,
                     numCol = NULL,
                     isHighTop = TRUE,
                     maxCutoffs = 5000,
                     mtMethod = c('BY', 'holm', 'hochberg',
                                  'hommel', 'bonferroni', 'BH',
                                  'fdr', 'none'),
                     pvalThr = NULL,
                     nCores = 1,
                     verbose = TRUE,
                     ...){
    mtMethod <- match.arg(mtMethod, c('BY', 'holm', 'hochberg',
                                      'hommel', 'bonferroni', 'BH',
                                      'fdr', 'none'))

    if (is.null(list3)){
        df <- expand.grid(names(list1), names(list2))
        if (is.null(universe2))
            type <- '2N' else
                type <- '2MN'
    } else {
        df <- expand.grid(names(list1), names(list2), names(list3))
        type <- '3N'
        if (!is.null(universe2))
            message('Three-way overlaps can be currently computed only for',
                    ' one universe. `universe2` will be ignored.')
    }
    colnames(df) <- paste0('Group', seq(ncol(df)))

    df$pval <- vapply(seq(nrow(df)),
                      function(i){
                          names1 <- df[i, 1]
                          names2 <- df[i, 2]
                          obj1 <- list1[[names1]]
                          obj2 <- list2[[names2]]
                          if(is.null(list3)){
                              obj3 <- NULL
                              if (verbose)
                                  message('Assessing overlap between sets ',
                                          names1, ' and ', names2, '...')
                          } else {
                              names3 <- df[i, 3]
                              obj3 <- list3[[names3]]
                              if (verbose)
                                  message('Assessing overlap between sets ',
                                          names1, ', ', names2, ' and ',
                                          names3, '...')
                          }
                          return(pvalObjects(obj1, obj2, obj3,
                                             universe1, universe2, numCol,
                                             isHighTop, maxCutoffs, mtMethod,
                                             nCores, type))
                      }, numeric(1))
    df <- mtCorrectDF(df, mtMethod, pvalThr=pvalThr, ...)
    return(df)
}

#' Build a Seurat marker list ready to be used by LISTO
#'
#' This function builds a Seurat marker list ready to be used by LISTO.
#' Requires Seurat (not automatically installed with LISTO).
#'
#' @param seuratObj A Seurat object.
#' @param col Seurat metadata column used for grouping.
#' @param logFCThr Fold change threshold for testing.
#' @param minPct The minimum fraction of in-cluster cells in which tested
#' genes need to be expressed.
#' @param ... Additional arguments passed to \code{Seurat::FindMarkers}.
#'
#' @return A list consisting of data frames generated with
#' \code{Seurat::FindMarkers}.
#'
#' @examples
#' seuratPath <- system.file('extdata', 'seuratObj.qs2', package='LISTO')
#' seuratObj <- qs2::qs_read(seuratPath)
#' a <- buildSeuratMarkerList(seuratObj, 'Cell_Cycle', logFCThr=0.1)
#'
#' @export
#'
buildSeuratMarkerList <- function(seuratObj,
                                  col,
                                  logFCThr = 1,
                                  minPct = 0.2,
                                  ...){
    if (requireNamespace('Seurat', quietly=TRUE)){
        groups <- unique(seuratObj[[]][[col]])
        markers <- lapply(groups,
                          function(x){
                              message('Computing markers for ', x, '...')
                              Seurat::FindMarkers(seuratObj,
                                                  group.by=col,
                                                  ident.1=x,
                                                  only.pos=TRUE,
                                                  min.pct=minPct,
                                                  logfc.threshold=logFCThr,
                                                  ...)
                               })
        names(markers) <- groups
        return(markers)
    } else
        message('Install Seurat to use `buildSeuratMarkerList`.')
}

#' Compute the probability that two subsets of sets M and N intersect in
#' at least k points
#'
#' This function computes the probability that two subsets A and B of sets
#' M and N intersect in at least k points.
#'
#' @inheritParams vNumeratorMN
#'
#' @return A numeric value in [0, 1] representing the probability that two
#' subsets of sets M and N intersect in at least k points.
#'
#' @examples
#' pvalCounts2MN (300, 23, 24, 6)
#'
#' @export
#'
pvalCounts2MN <- function(intMN, intAN, intBM, k){
    denom <- -1 * vChoose(intMN, intBM)
    pval <- sum(vapply(seq(k, min(intAN, intBM)), function(i){
        exponents <- vSum(vNumeratorMN(intMN, intAN, intBM, i), denom)
        primes <- generate_n_primes(length(exponents))
        return(powerProduct(primes, exponents))
    }, numeric(1)))
    return(pval)
}

#' Compute the probability that three subsets of a set intersect in at least k points
#'
#' This function computes the probability that three subsets of a set intersect
#' in at least k points.
#'
#' @param lenA Size of the first subset.
#' @param lenB Size of the second subset.
#' @param lenC Size of the third subset.
#' @param n Size of the set comprising the subsets.
#' @param k Size of the intersection.
#'
#' @return A numeric value in [0, 1] representing the probability that
#' three subsets of a set intersect in at least k points.
#'
#' @examples
#' pvalCounts3N (300, 200, 250, 400, 180)
#'
#' @export
#'
pvalCounts3N <- function(lenA, lenB, lenC, n, k){
    if (k == 0)
        return(1)
    lengths <- sort(c(lenA, lenB, lenC))
    if (k > lengths[[1]])
        stop('`k` must not exceed the minimum length of the three sets.')
    return (sum(vapply(seq(k, lengths[[1]]),
                       function(x) probCounts3N(lengths[[1]],
                                                lengths[[2]],
                                                lengths[[3]],
                                                n,
                                                x), numeric(1))))
}

#' Extract numeric values from an input object
#'
#' This function extracts numeric values from an input object.
#'
#' @param obj A data frame with a numeric column, or a character vector.
#' @param numCol The name of the numeric column used for data frame ordering.
#' @param isHighTop Whether higher values in the numeric column correspond to
#' top-ranked items.
#'
#' @keywords internal
#'
getObjectValues <- function(obj, numCol = NULL, isHighTop = TRUE){
    if (is.null(numCol) | is(obj, 'character')){
        if(isHighTop)
            return (-Inf)
        return(Inf)
    }
    return(obj[[numCol]])
}

#' Generate cutoffs for filtering overlaps
#'
#' This function generates cutoffs for filtering overlaps
#'
#' @inheritParams getObjectValues
#' @param obj1 A data frame with a numeric column, or a character vector.
#' @param obj2 A data frame with a numeric column, or a character vector.
#' @param obj3 A data frame with a numeric column, or a character vector.
#' @param maxCutoffs Maximum number of cutoffs. If the input data frames
#' contain more cutoffs than this value, only \code{maxCutoffs} linearly
#' spaced cutoffs will be selected from the generated cutoff list.
#'
#' @return A numeric vector.
#'
#' @keywords internal
#'
generateCutoffs <- function(obj1,
                            obj2,
                            obj3 = NULL,
                            numCol = NULL,
                            isHighTop = TRUE,
                            maxCutoffs = 5000){

    values1 <- getObjectValues(obj1, numCol, isHighTop)
    values2 <- getObjectValues(obj2, numCol, isHighTop)
    if(is.null(obj3)){
        values3 <- values2
        cutoffs <- unique(c(values1, values2))
    } else {
        values3 <- getObjectValues(obj3, numCol, isHighTop)
        cutoffs <- unique(c(values1, values2, values3))
    }

    if (isHighTop){
        bound <- min(max(values1), max(values2), max(values3))
        cutoffs <- cutoffs[cutoffs < bound]
    } else{
        bound <- max(min(values1), min(values2), min(values3))
        cutoffs <- cutoffs[cutoffs > bound]
    }

    cutoffs <- sort(cutoffs, decreasing=isHighTop)
    extraCutoff <- (1 - 2 * as.integer(isHighTop)) * Inf
    cutoffs <- unique(c(cutoffs, extraCutoff))
    nCutoffs <- length(cutoffs)
    if (nCutoffs > maxCutoffs){
        message(nCutoffs, ' cutoffs found in the input data frames. Only ',
                maxCutoffs, ' will be used. To change this behavior, set a ',
                'higher value to `maxCutoffs`.')
        cutoffs <- cutoffs[seq(1, nCutoffs, length.out=maxCutoffs)]
    }
    return(cutoffs)
}

#' @importFrom primes generate_n_primes generate_primes
#' @importFrom statisfactory psum
#'
NULL

#' Compute the probability that two subsets of sets M and N intersect in k
#' points
#'
#' This function computes the probability that two subsets of sets M and N
#' intersect in k points. Intersection sizes (M with N, A with N and B with M)
#' must be provided.
#'
#' @inheritParams vNumeratorMN
#'
#' @return A numeric value in [0, 1] representing the probability that two
#' subsets of sets M and N intersect in k points.
#'
#' @examples
#' probCounts2MN(8, 6, 4, 2)
#'
#' @export
#'
probCounts2MN <- function(intMN, intAN, intBM, k){
    exponents <- vSum(vNumeratorMN(intMN, intAN, intBM, k),
                      -1 * vChoose(intMN, intBM))
    primes <- generate_n_primes(length(exponents))
    return(powerProduct(primes, exponents))
}

#' Compute the probability that three subsets of given sizes intersect in k
#' points
#'
#' This function computes the probability that three subsets of given sizes
#' intersect in k points.
#'
#' @param a Size of the first subset.
#' @param b Size of the second subset.
#' @param c Size of the third subset.
#' @param n Size of the set.
#' @param k Size of the intersection.
#'
#' @return A numeric value in [0, 1] representing the probability that three
#' subsets of given sizes intersect in k points.
#'
#' @examples
#' probCounts3N(8, 6, 10, 20, 3)
#'
#' @export
#'
probCounts3N <- function(a, b, c, n, k){
    v <- seq(max(a + b - n, k), min(a, b, n + k - c))
    prob <- sum(vapply(v, function(x) dhyper(x, a, n - a, b) *
                            dhyper(k, x, n - x, c), numeric(1)))
    return (prob)
}

#' Check if numCol is valid input for an object.
#'
#' This function checks if \code{numCol} is valid input for an object.
#'
#' @inheritParams getObjectValues
#'
#' @return Nothing. This function is called for its side effect.
#'
#' @noRd
#'
checkNumCol <- function(obj, numCol){
    if(is(obj, 'data.frame')){
        if (!numCol %in% colnames(obj))
            stop('All data frames must contain a `numCol` column.')
        if (!is(obj[, numCol], 'numeric'))
            stop('The `numCol` column must be numeric in all data frames.')
    }

}

#' Check if numCol is valid input for a list of objects
#'
#' This function checks if \code{numCol} is valid input for a list of objects.
#'
#' @inheritParams getObjectValues
#' @param objs A list containing data frame with a numeric column
#' or character vectors.
#'
#' @return Nothing. This function is called for its side effect.
#'
#' @noRd
#'
checkNumColAll <- function(objs, numCol)
    for (obj in objs)
        checkNumCol(obj, numCol)

#' Calculate the p-value of intersection for two sets
#'
#' This function calculates the p-value of intersection for two sets.
#'
#' @details A thin wrapper around \code{stats::phyper}.
#'
#' @param a A character vector.
#' @param b A character vector.
#' @param n Set from which \code{a} and \code{b} are
#' selected.
#'
#' @return A numeric value in [0, 1] representing the p-value of intersection
#' for two sets.
#'
#' @examples
#' pvalSets2N(LETTERS[seq(4, 10)], LETTERS[seq(7, 15)], LETTERS)
#'
#' @export
#'
pvalSets2N <- function(a, b, n){
    na <- length(a)
    nb <- length(b)
    nShared <- length(intersect(a, b))
    nn <- length(n)
    return (phyper(nShared - 1,
                   na,
                   nn - na,
                   nb,
                   lower.tail=FALSE))
}

#' Compute the p-value of intersection of two subsets of sets M and N
#'
#' This function computes the p-value of intersection of two subsets
#' of sets M and N.
#'
#' @details A thin wrapper around \code{pvalCounts2MN}.
#'
#' @inheritParams pvalSets2N
#' @param m Set from which \code{a} is selected.
#' @param n Set from which \code{b} is selected.
#'
#' @return A numeric value in [0, 1] representing the p-value of intersection
#' of two subsets of sets M and N.
#'
#' @examples
#' pvalSets2MN(LETTERS[seq(4, 10)],
#' LETTERS[seq(7, 15)],
#' LETTERS[seq(19)],
#' LETTERS[seq(6, 26)])
#'
#' @export
#'
pvalSets2MN <- function(a, b, m, n){
    if (length(setdiff(a, m)))
        stop('`a` must be a subset of `m`.')
    if (length(setdiff(b, n)))
        stop('`b` must be a subset of `n`.')
    return(pvalCounts2MN(length(intersect(m, n)),
                         length(intersect(a, n)),
                         length(intersect(b, m)),
                         length(intersect(a, b))
    ))
}

#' Compute the p-value of intersection of three subsets
#'
#' This function computes the p-value of intersection of three subsets.
#'
#' @details A thin wrapper around \code{pvalCounts3N}.
#'
#' @inheritParams pvalSets2N
#' @param c A character vector.
#' @param n Set from which \code{a}, \code{b} and \code{c} are
#' selected.
#'
#' @return A numeric value in [0, 1] representing the p-value of intersection
#' of three subsets.
#'
#' @examples
#' pvalSets3N(LETTERS[seq(4, 10)],
#' LETTERS[seq(7, 15)],
#' LETTERS[seq(19)],
#' LETTERS)
#'
#' @export
#'
pvalSets3N <- function(a, b, c, n){
    if (length(setdiff(a, n)))
        stop('`a` must be a subset of `n`.')
    if (length(setdiff(b, n)))
        stop('`b` must be a subset of `n`.')
    if (length(setdiff(c, n)))
        stop('`c` must be a subset of `n`.')
    return(pvalCounts3N(length(a),
                        length(b),
                        length(c),
                        length(n),
                        length(Reduce(intersect, list(a, b, c)))))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_listo.R
\name{runLISTO}
\alias{runLISTO}
\title{Assess the overlap of two or three lists of objects.}
\usage{
runLISTO(
  list1,
  list2,
  list3 = NULL,
  universe1,
  universe2 = NULL,
  numCol = NULL,
  isHighTop = TRUE,
  maxCutoffs = 5000,
  mtMethod = c("BY", "holm", "hochberg", "hommel", "bonferroni", "BH", "fdr", "none"),
  pvalThr = NULL,
  nCores = 1,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{list1}{A list containing character vectors, or data frames having
a numeric column.}

\item{list2}{A list containing character vectors, or data frames having a
numeric column.}

\item{list3}{A list containing character vectors, or data frames having a
numeric column.}

\item{universe1}{Character vector; the set from which the items
corresponding to the elements in \code{list1} are selected.}

\item{universe2}{Character vector; the set from which the items
corresponding to the elements in \code{list2} are selected.}

\item{numCol}{The name of the numeric column used for data frame ordering.}

\item{isHighTop}{Whether higher values in the numeric column correspond to
top-ranked items.}

\item{maxCutoffs}{Maximum number of cutoffs. If the input data frames
contain more cutoffs than this value, only \code{maxCutoffs} linearly
spaced cutoffs will be selected from the generated cutoff list.}

\item{mtMethod}{Multiple testing correction method.}

\item{pvalThr}{Threshold to filter the results based on the adjusted
p-values. If \code{NULL} as default, no filtering will be performed.}

\item{nCores}{Number of cores. If performing an overlap assessment between
sets belonging to the same universe, it is recommended not to use
parallelization (that is, leave this parameter as 1).}

\item{verbose}{Logical; whether the output should be verbose.}

\item{...}{Additional arguments passed to \code{mtCorrectDF}.}
}
\value{
A data frame listing the p-value and adjusted p-value for each
overlap. Combinations of overlaps are represented through the first two
(or three if \code{list3} is not \code{NULL}) columns, while the penultimate
column records the overlap p-values and the last column records the adjusted
overlap p-values.
}
\description{
This function assesses the overlap of two or three lists of objects
(character vectors, or data frames having at least one numeric column).
}
\examples{
donorPath <- system.file('extdata', 'donorMarkers.qs2', package='LISTO')
donorMarkers <- qs2::qs_read(donorPath)[seq(3)]
labelPath <- system.file('extdata', 'labelMarkers.qs2', package='LISTO')
labelMarkers <- qs2::qs_read(labelPath)[seq(3)]
universe1Path <- system.file('extdata', 'universe1.qs2', package='LISTO')
universe1 <- qs2::qs_read(universe1Path)
res <-  runLISTO(donorMarkers, labelMarkers, universe1=universe1,
numCol='avg_log2FC')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seurat_demo.R
\name{buildSeuratMarkerList}
\alias{buildSeuratMarkerList}
\title{Build a Seurat marker list ready to be used by LISTO}
\usage{
buildSeuratMarkerList(seuratObj, col, logFCThr = 1, minPct = 0.2, ...)
}
\arguments{
\item{seuratObj}{A Seurat object.}

\item{col}{Seurat metadata column used for grouping.}

\item{logFCThr}{Fold change threshold for testing.}

\item{minPct}{The minimum fraction of in-cluster cells in which tested
genes need to be expressed.}

\item{...}{Additional arguments passed to \code{Seurat::FindMarkers}.}
}
\value{
A list consisting of data frames generated with
\code{Seurat::FindMarkers}.
}
\description{
This function builds a Seurat marker list ready to be used by LISTO.
Requires Seurat (not automatically installed with LISTO).
}
\examples{
seuratPath <- system.file('extdata', 'seuratObj.qs2', package='LISTO')
seuratObj <- qs2::qs_read(seuratPath)
a <- buildSeuratMarkerList(seuratObj, 'Cell_Cycle', logFCThr=0.1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiple_testing.R
\name{mtCorrectV}
\alias{mtCorrectV}
\title{Perform multiple testing correction on a vector of p-values}
\usage{
mtCorrectV(
  pvals,
  mtMethod = c("BY", "holm", "hochberg", "hommel", "bonferroni", "BH", "fdr", "none"),
  mtStat = c("identity", "median", "mean", "max", "min"),
  nComp = length(pvals)
)
}
\arguments{
\item{pvals}{A numeric vector.}

\item{mtMethod}{Multiple testing correction method. Choices are
'BY' (default) 'holm', hochberg', hommel', 'bonferroni', 'BH',  'fdr' and
'none'.}

\item{mtStat}{A statistics to be optionally computed. Choices are 'identity'
(no statistics will be computed and the adjusted p-values will be returned
as such), 'median', 'mean', 'max' and 'min'.}

\item{nComp}{Number of comparisons. In most situations, this parameter
should not be changed.}
}
\value{
If \code{mtStat} is 'identity' (as default), a numeric vector of
p-values corrected for multiple testing. Otherwise, a statistic based on
these corrected p-values defined by \code{mtStat}.
}
\description{
This function performs multiple testing correction on a vector of p-values.
}
\examples{
pvals <- c(0.032, 0.001, 0.0045, 0.051, 0.048)
mtCorrectV(pvals)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_objects.R
\name{pvalObjectsCore}
\alias{pvalObjectsCore}
\title{Compute the p-value of overlap for two or three objects}
\usage{
pvalObjectsCore(
  obj1,
  obj2,
  obj3 = NULL,
  universe1,
  universe2 = NULL,
  numCol = NULL,
  cutoff = NULL,
  compFun = `>`,
  type = c("2N", "2MN", "3N")
)
}
\arguments{
\item{obj1}{A data frame with a numeric column, or a character vector.}

\item{obj2}{A data frame with a numeric column, or a character vector.}

\item{obj3}{A data frame with a numeric column, or a character vector.}

\item{universe1}{The set from which the items stored
in \code{obj1} are selected.}

\item{universe2}{The set from which the items stored
in \code{obj2} are selected.}

\item{numCol}{The name of the numeric column used for data frame ordering.}
}
\value{
A p-value.
}
\description{
This function computes the p-value of overlap for two or three objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiple_testing.R
\name{mtCorrectDF}
\alias{mtCorrectDF}
\title{Perform multiple testing correction on a data frame column}
\usage{
mtCorrectDF(
  df,
  mtMethod = c("BY", "holm", "hochberg", "hommel", "bonferroni", "BH", "fdr", "none"),
  colStr = "pval",
  newColStr = "pvalAdj",
  pvalThr = 0.05,
  doOrder = TRUE,
  nComp = nrow(df)
)
}
\arguments{
\item{df}{A data frame with a p-values column.}

\item{mtMethod}{Multiple testing correction method. Choices are
'BY' (default) 'holm', hochberg', hommel', 'bonferroni', 'BH',  'fdr' and
'none'.}

\item{colStr}{Name of the column of p-values.}

\item{newColStr}{Name of the column of adjusted p-values that will be
created.}

\item{pvalThr}{p-value threshold used for filtering. If \code{NULL}, no
filtering will be performed.}

\item{doOrder}{Whether to increasingly order the data frame based on the
adjusted p-values.}

\item{nComp}{Number of comparisons. In most situations, this parameter
should not be changed.}
}
\value{
A data frame with the p-value column corrected for multiple testing.
}
\description{
This function orders a data frame based on a column of p-values, performs
multiple testing on the column, and filters the data-frame based on it.
}
\examples{
df <- data.frame(elem = c('A', 'B', 'C', 'D', 'E'),
pval = c(0.032, 0.001, 0.0045, 0.051, 0.048))
mtCorrectDF(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_objects.R
\name{pvalObjects}
\alias{pvalObjects}
\title{Assess the overlap of two or three objects}
\usage{
pvalObjects(
  obj1,
  obj2,
  obj3 = NULL,
  universe1,
  universe2 = NULL,
  numCol = NULL,
  isHighTop = TRUE,
  maxCutoffs = 5000,
  mtMethod = c("BY", "holm", "hochberg", "hommel", "bonferroni", "BH", "fdr", "none"),
  nCores = 1,
  type = c("2N", "2MN", "3N")
)
}
\arguments{
\item{obj1}{A data frame with a numeric column, or a character vector.}

\item{obj2}{A data frame with a numeric column, or a character vector.}

\item{obj3}{A data frame with a numeric column, or a character vector.}

\item{universe1}{The set from which the items stored
in \code{obj1} are selected.}

\item{universe2}{The set from which the items stored
in \code{obj2} are selected.}

\item{numCol}{The name of the numeric column used for data frame ordering.}

\item{isHighTop}{Whether higher values in the numeric column correspond to
top-ranked items.}

\item{maxCutoffs}{Maximum number of cutoffs. If the input data frames
contain more cutoffs than this value, only \code{maxCutoffs} linearly
spaced cutoffs will be selected from the generated cutoff list.}

\item{mtMethod}{Multiple testing correction method.}

\item{nCores}{Number of cores. If performing an overlap assessment between
sets belonging to the same universe, it is recommended not to use
parallelization (that is, leave this parameter as 1).}

\item{type}{Type of overlap assessment. Choose between: two sets belonging
to the same universe ('2N'), two sets belonging to different universes
('2MN'), three sets belonging to the same universe ('3MN').}
}
\value{
A numeric value in [0, 1] representing the p-value of the
overlap of the two objects.
}
\description{
This function assesses the overlap of two or three objects
(character vectors, or data frames having a numeric column).
}
\examples{
pvalObjects(LETTERS[seq(2, 7)], LETTERS[seq(3, 19)], universe1=LETTERS)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_sets.R
\name{pvalSets3N}
\alias{pvalSets3N}
\title{Compute the p-value of intersection of three subsets}
\usage{
pvalSets3N(a, b, c, n)
}
\arguments{
\item{a}{A character vector.}

\item{b}{A character vector.}

\item{c}{A character vector.}

\item{n}{Set from which \code{a}, \code{b} and \code{c} are
selected.}
}
\value{
A numeric value in [0, 1] representing the p-value of intersection
of three subsets.
}
\description{
This function computes the p-value of intersection of three subsets.
}
\details{
A thin wrapper around \code{pvalCounts3N}.
}
\examples{
pvalSets3N(LETTERS[seq(4, 10)],
LETTERS[seq(7, 15)],
LETTERS[seq(19)],
LETTERS)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiple_testing.R
\name{bfCorrectV}
\alias{bfCorrectV}
\title{Perform Bonferroni correction on a vector of p-values}
\usage{
bfCorrectV(pvals, nComp)
}
\arguments{
\item{pvals}{A numeric vector.}

\item{nComp}{Number of comparisons. In most situations, this parameter
should not be changed.}
}
\value{
Bonferroni-corrected p-values
}
\description{
This function performs Bonferroni correction on a vector of p-values.
}
\details{
This function is implemented in order to allow users to perform
Bonferroni correction with fewer comparisons than the number of elements
in the vector, which is normally disallowed by the \code{p.adjust} function
from \code{stats}. A use case is correcting the Seurat markers returned
for each identity class by setting the number of comparisons to the number
of classes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutoffs.R
\name{generateCutoffs}
\alias{generateCutoffs}
\title{Generate cutoffs for filtering overlaps}
\usage{
generateCutoffs(
  obj1,
  obj2,
  obj3 = NULL,
  numCol = NULL,
  isHighTop = TRUE,
  maxCutoffs = 5000
)
}
\arguments{
\item{obj1}{A data frame with a numeric column, or a character vector.}

\item{obj2}{A data frame with a numeric column, or a character vector.}

\item{obj3}{A data frame with a numeric column, or a character vector.}

\item{numCol}{The name of the numeric column used for data frame ordering.}

\item{isHighTop}{Whether higher values in the numeric column correspond to
top-ranked items.}

\item{maxCutoffs}{Maximum number of cutoffs. If the input data frames
contain more cutoffs than this value, only \code{maxCutoffs} linearly
spaced cutoffs will be selected from the generated cutoff list.}
}
\value{
A numeric vector.
}
\description{
This function generates cutoffs for filtering overlaps
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vectors.R
\name{vSum}
\alias{vSum}
\title{Add numeric vectors of different lenghts}
\usage{
vSum(...)
}
\arguments{
\item{...}{Numeric vectors.}
}
\value{
A numeric vector.
}
\description{
This function adds numeric vectors of different lengths by filling shorter
vectors with zeroes.
}
\examples{
vSum(c(1, 4), c(2, 8, 6), c(1, 7), c(10, 4, 6, 7))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_counts.R
\name{pvalCounts2MN}
\alias{pvalCounts2MN}
\title{Compute the probability that two subsets of sets M and N intersect in
at least k points}
\usage{
pvalCounts2MN(intMN, intAN, intBM, k)
}
\arguments{
\item{intMN}{Number of elements in the intersection of sets M and N.}

\item{intAN}{Number of elements in the intersection of sets A (subset of M)
and N.}

\item{intBM}{Number of elements in the intersection of sets B (subset of N)
and M.}

\item{k}{Number of elements in the intersection of sets A and B.}
}
\value{
A numeric value in [0, 1] representing the probability that two
subsets of sets M and N intersect in at least k points.
}
\description{
This function computes the probability that two subsets A and B of sets
M and N intersect in at least k points.
}
\examples{
pvalCounts2MN (300, 23, 24, 6)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vectors.R
\name{vChoose}
\alias{vChoose}
\title{Compute the prime factor decomposition of the binomial coefficient}
\usage{
vChoose(n, k)
}
\arguments{
\item{n}{Total number of elements.}

\item{k}{Number of selected elements.}
}
\value{
A vector in which positions represent prime numbers (that is, the
first position corresponds to 2, the second position corresponds to 3,
the third position corresponds to 5, etc.) and values
represent their exponents in the factorial decomposition.
}
\description{
This function computes the prime factor decomposition of the
binomial coefficient.
}
\examples{
vChoose(8, 4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vectors.R
\name{vNumeratorMN}
\alias{vNumeratorMN}
\title{Compute the prime representation of the numerator of the fraction
representing the probability that two subsets of sets M and N intersect
in k points}
\usage{
vNumeratorMN(intMN, intAN, intBM, k)
}
\arguments{
\item{intMN}{Number of elements in the intersection of sets M and N.}

\item{intAN}{Number of elements in the intersection of sets A (subset of M)
and N.}

\item{intBM}{Number of elements in the intersection of sets B (subset of N)
and M.}

\item{k}{Number of elements in the intersection of sets A and B.}
}
\value{
A vector containing the prime representation of the fraction
representing the probability that two subsets of sets M and N intersect in k
points. Positions represent prime numbers in order (2, 3, 5...), and values
represent their exponents in the prime decomposition.
}
\description{
This function computes the numerator of the fraction representing the
probability that two subsets of sets M and N intersect in k points
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prob_counts.R
\name{probCounts3N}
\alias{probCounts3N}
\title{Compute the probability that three subsets of given sizes intersect in k
points}
\usage{
probCounts3N(a, b, c, n, k)
}
\arguments{
\item{a}{Size of the first subset.}

\item{b}{Size of the second subset.}

\item{c}{Size of the third subset.}

\item{n}{Size of the set.}

\item{k}{Size of the intersection.}
}
\value{
A numeric value in [0, 1] representing the probability that three
subsets of given sizes intersect in k points.
}
\description{
This function computes the probability that three subsets of given sizes
intersect in k points.
}
\examples{
probCounts3N(8, 6, 10, 20, 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_objects.R
\name{filterItems}
\alias{filterItems}
\title{Filter items based on a provided cutoff}
\usage{
filterItems(obj, numCol = NULL, cutoff = NULL, compFun = `>`)
}
\arguments{
\item{obj}{A data frame with a numeric column, or a character vector.}

\item{numCol}{The name of the numeric column used for data frame ordering.}

\item{cutoff}{Cutoff for assessing item overlaps.}

\item{compFun}{Comparison function.}
}
\description{
This function filters items based on a provided cutoff.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_sets.R
\name{pvalSets2N}
\alias{pvalSets2N}
\title{Calculate the p-value of intersection for two sets}
\usage{
pvalSets2N(a, b, n)
}
\arguments{
\item{a}{A character vector.}

\item{b}{A character vector.}

\item{n}{Set from which \code{a} and \code{b} are
selected.}
}
\value{
A numeric value in [0, 1] representing the p-value of intersection
for two sets.
}
\description{
This function calculates the p-value of intersection for two sets.
}
\details{
A thin wrapper around \code{stats::phyper}.
}
\examples{
pvalSets2N(LETTERS[seq(4, 10)], LETTERS[seq(7, 15)], LETTERS)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factor_decomposition.R
\name{factorialPrimePowers}
\alias{factorialPrimePowers}
\title{Generate the prime factor decomposition of n factorial.}
\usage{
factorialPrimePowers(n)
}
\arguments{
\item{n}{A positive integer.}
}
\value{
A vector in which positions represent prime numbers (that is, the
first position corresponds to 2, the second position corresponds to 3,
the third position corresponds to 5, etc.) and values
represent their exponents in the factorial decomposition.
}
\description{
This function generates the prime factor decomposition of n factorial.
}
\examples{
factorialPrimePowers(8)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiple_testing.R
\name{mtCorrectHelper}
\alias{mtCorrectHelper}
\title{Helper function for multiple comparison testing}
\usage{
mtCorrectHelper(pvals, mtMethod, nComp)
}
\arguments{
\item{pvals}{A numeric vector.}

\item{mtMethod}{Multiple testing correction method. Choices are
'BY' (default) 'holm', hochberg', hommel', 'bonferroni', 'BH',  'fdr' and
'none'.}

\item{nComp}{Number of comparisons. In most situations, this parameter
should not be changed.}
}
\value{
Adjusted p-values.
}
\description{
This function is a helper for multiple comparison testing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_counts.R
\name{pvalCounts3N}
\alias{pvalCounts3N}
\title{Compute the probability that three subsets of a set intersect in at least k points}
\usage{
pvalCounts3N(lenA, lenB, lenC, n, k)
}
\arguments{
\item{lenA}{Size of the first subset.}

\item{lenB}{Size of the second subset.}

\item{lenC}{Size of the third subset.}

\item{n}{Size of the set comprising the subsets.}

\item{k}{Size of the intersection.}
}
\value{
A numeric value in [0, 1] representing the probability that
three subsets of a set intersect in at least k points.
}
\description{
This function computes the probability that three subsets of a set intersect
in at least k points.
}
\examples{
pvalCounts3N (300, 200, 250, 400, 180)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutoffs.R
\name{getObjectValues}
\alias{getObjectValues}
\title{Extract numeric values from an input object}
\usage{
getObjectValues(obj, numCol = NULL, isHighTop = TRUE)
}
\arguments{
\item{obj}{A data frame with a numeric column, or a character vector.}

\item{numCol}{The name of the numeric column used for data frame ordering.}

\item{isHighTop}{Whether higher values in the numeric column correspond to
top-ranked items.}
}
\description{
This function extracts numeric values from an input object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pval_sets.R
\name{pvalSets2MN}
\alias{pvalSets2MN}
\title{Compute the p-value of intersection of two subsets of sets M and N}
\usage{
pvalSets2MN(a, b, m, n)
}
\arguments{
\item{a}{A character vector.}

\item{b}{A character vector.}

\item{m}{Set from which \code{a} is selected.}

\item{n}{Set from which \code{b} is selected.}
}
\value{
A numeric value in [0, 1] representing the p-value of intersection
of two subsets of sets M and N.
}
\description{
This function computes the p-value of intersection of two subsets
of sets M and N.
}
\details{
A thin wrapper around \code{pvalCounts2MN}.
}
\examples{
pvalSets2MN(LETTERS[seq(4, 10)],
LETTERS[seq(7, 15)],
LETTERS[seq(19)],
LETTERS[seq(6, 26)])

}
